@@ -105,6 +105,51 @@ else :
 		centroids = cent
 		return ( labels, centroids )
 
+from scipy.spatial.distance import squareform , pdist
+absolute_coordinates_to_distance_matrix = lambda Q:squareform(pdist(Q))
+
+distance_matrix_to_geometry_conversion_notes = """
+*) TAKE NOTE THAT THE OLD ALGORITHM CALLED DISTANCE GEOMETRY EXISTS. IT CAN BE EMPLOYED TO ANY DIMENSIONAL DATA. HERE YOU FIND A SVD BASED ANALOG OF THAT OLD METHOD.
+
+*) PDIST REALLY LIKES TO COMPUTE SQUARE ROOT OF THINGS SO WE SQUARE THE RESULT IF IT IS NOT SQUARED.
+
+*) IN SHORT THE DISTANCE MATRIX IN THE CONVERSION ROUTINE BACK TO ABSOLUTE COORDINATES USES R2 DISTANCES.
+"""
+
+if bUseNumba :
+	@jit(nopython=True)
+	def distance_matrix_to_absolute_coordinates ( D , bSquared = False, n_dimensions=2 ):
+		if not bSquared :
+			D = D**2.
+		DIM = n_dimensions
+		DIJ = D*0.
+		M = len(D)
+		for i in range(M) :
+			for j in range(M) :
+				DIJ[i,j] = 0.5* (D[i,-1]+D[j,-1]-D[i,j])
+		D = DIJ
+		U,S,Vt = np.linalg.svd ( D , full_matrices = True )
+		S[DIM:] *= 0.
+		Z = np.diag(S**0.5)[:,:DIM]
+		xr = np.dot( Z.T,Vt )
+		return ( xr )
+else :
+        def distance_matrix_to_absolute_coordinates ( D , bSquared = False, n_dimensions=2 ):
+                if not bSquared :
+                        D = D**2.
+                DIM = n_dimensions
+                DIJ = D*0.
+                M = len(D)
+                for i in range(M) :
+                        for j in range(M) :
+                                DIJ[i,j] = 0.5* (D[i,-1]+D[j,-1]-D[i,j])
+                D = DIJ
+                U,S,Vt = np.linalg.svd ( D , full_matrices = True )
+                S[DIM:] *= 0.
+                Z = np.diag(S**0.5)[:,:DIM]
+                xr = np.dot( Z.T,Vt )
+                return ( xr )
+
 
 def connectivity ( B , val, bVerbose=True ) :
 	description="""
@@ -289,6 +334,26 @@ def run_clustering_and_write_gmt( df , ca , filename = './approx_cluster_file.gm
             print ( 'CLU-'+str(ulab),'\tDESCRIPTION\t'+'\t'.join(analytes), file=of )
 
 
+def projection_knn_assignment ( projected_coords , df , NMaxGuess=-1 , n_dimensions=2  ) :
+    coords_s = projected_coords.dropna( 0 )
+    centroid_coordinates = []
+    for row in df.T :
+        guess = sorted ( [ (v,i) for (v,i) in zip( df.loc[row].values,df.loc[row].index ) ] ) [::-1][:NMaxGuess]
+        maxWeights = [ i[1] for i in guess ]
+        use = df.loc[row,maxWeights]
+        S = np.sum ( use.values )
+        S = 1. if S==0 else S
+        crd = np.dot(use.values,coords_s.loc[use.index.values].values)/S
+        centroid_coordinates.append(crd)
+
+    centroids_df = pd.DataFrame ( centroid_coordinates , index=df.index , columns=[ 'C'+str(i) for i in range(n_dimensions) ] )
+    labels , centroids = seeded_kmeans( coords_s.values,centroids_df.values )
+    coords_s.loc[:,'owner'] = centroids_df.iloc[labels].index.values
+    for i in range(len(centroids.T)) :
+        centroids_df.loc[:,'E'+str(i) ] = (centroids.T)[i]
+    return ( centroids_df , coords_s )
+
+
 def make_clustering_visualisation_df ( CLUSTER , df=None , add_synonyms = False ,
                                     output_name = 'feature_clusters_output.csv' 
                                   ) :
@@ -321,9 +386,43 @@ def make_clustering_visualisation_df ( CLUSTER , df=None , add_synonyms = False
     clustering_df.to_csv( output_name , '\t' )
     return ( clustering_df )
 
+def backprojection_clustering ( analyte_df , bRanked=False , n_dimensions=2 ,
+                                bDoFeatures=True , bDoSamples=True ) :
+    from scipy.stats import rankdata
+    if bRanked :
+        rana_df = analyte_df .apply( lambda x:(rankdata(x,'average')-0.5)/len(x) )
+    else :
+        rana_df = analyte_df
 
-from scipy.spatial.distance import squareform , pdist
-abolsute_coordinates_to_distance_matrix = lambda Q:squareform(pdist(Q))
+    dimcrdnames = [ 'd'+str(i) for i in range(n_dimensions) ]
+    #
+    # Do backprojection clustering with backprojection
+    cluster_coords_f = None
+    if bDoFeatures :
+        #
+        dM1 = absolute_coordinates_to_distance_matrix( rana_df.values   )
+        pd.DataFrame(dM1,index=rana_df.index,columns=rana_df.index).to_csv('../data/dM1.tsv','\t')
+        #
+        # Project it back onto first two components
+        max_var_projection = distance_matrix_to_absolute_coordinates ( dM1 , n_dimensions=n_dimensions )
+        cluster_coords_f = pd.DataFrame( max_var_projection ,
+                                    columns = rana_df.index ,
+                                    index = dimcrdnames ).T
+    cluster_coords_s = None
+    if bDoSamples :
+        #
+        # And again for all the samples
+        dM2 = absolute_coordinates_to_distance_matrix( rana_df.T.values )
+        pd.DataFrame(dM2,index=rana_df.columns,columns=rana_df.columns).to_csv('../data/dM2.tsv','\t')
+        #
+        # This algorithm is exact but scales somewhere between n^2 and n log n
+        max_var_projection = distance_matrix_to_absolute_coordinates ( dM2 , n_dimensions=n_dimensions )
+        cluster_coords_s = pd.DataFrame( max_var_projection ,
+                                    columns = rana_df.columns ,
+                                    index = dimcrdnames ).T
+        cluster_coords_s.to_csv('../data/conclust_s.tsv','\t')
+
+    return ( cluster_coords_f,cluster_coords_s )
 
 
 if __name__ == '__main__' :
