@@ -173,12 +173,15 @@ def calculate_alignment_properties ( encoding_df , quantx, quanty, scorex,
     #
     use_centroids = list(  quanty[use_centroid_indices]  )
     use_labels    = list( encoding_df.columns.values[use_centroid_indices] )  
-    
+    #
     if owner_by == 'tesselation' :
         transcript_owner = [ use_labels[ np.argmin([ np.sum((xw-cent)**2) for cent in use_centroids ])] for xw in quantx ]
+        sample_owner     = [ use_labels[ np.argmin([ np.sum((yw-cent)**2) for cent in use_centroids ])] for yw in scorex ]
+    #
     if owner_by == 'angle' :
-        anglular_proximity = lambda B,A : 1 - np.dot(A,B) / ( np.sqrt(np.dot(A,A))*np.sqrt(np.dot(B,B)) )
-        transcript_owner   = [ use_labels[ np.argmin([ anglular_proximity(xw,cent) for cent in use_centroids ])] for xw in quantx ]
+        angular_proximity = lambda B,A : 1 - np.dot(A,B) / ( np.sqrt(np.dot(A,A))*np.sqrt(np.dot(B,B)) )
+        transcript_owner = [ use_labels[ np.argmin([ angular_proximity(xw,cent) for cent in use_centroids ])] for xw in quantx ]
+        sample_owner = [ use_labels[ np.argmin([ angular_proximity(yw,cent) for cent in use_centroids ])] for yw in scorex ]
     #
     # print ( 'PLS WEIGHT RADIUS' )
     radius  = lambda vector:np.sqrt(np.sum((vector)**2)) # radii
@@ -254,10 +257,7 @@ def calculate_alignment_properties ( encoding_df , quantx, quanty, scorex,
             else :
                 qdf['alpha'] = [ '0.3' for a in transcript_owner ]
         else :
-            if '_' in ''.join(names) :
-                qdf['owner'] = [ n.split('_')[0] for n in names ]
-            else:
-                qdf['owner'] = names
+            qdf['owner'] = sample_owner # The default should be the aligned projection weight
             qdf['alpha'] = [ '0.2' for n in names ]
         if synonyms is None :
             qdf['name'] = names
@@ -298,9 +298,8 @@ def run_rpls_regression ( analyte_df , journal_df , formula ,
 
 import impetuous.fit as ifit
 import impetuous.clustering as icluster
-def run_shape_alignment_clustering ( analyte_df , journal_df , formula, bVerbose = False ):
-
-	NOTE_ = "This is just a standard kmeans in arbitrary dimensions that start out with centroids that have been shape aligned"
+def run_shape_alignment_clustering ( analyte_df , journal_df , formula, bVerbose = False ) :
+	NOTE_ = "This is just a kmeans in arbitrary dimensions that start out with centroids that have been shape aligned"
 	encoding_df = interpret_problem ( analyte_df , journal_df , formula , bVerbose = bVerbose )
 
 	Q = encoding_df.T.apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).values
@@ -329,6 +328,40 @@ def run_shape_alignment_clustering ( analyte_df , journal_df , formula, bVerbose
 
 	return ( res_df , clusters_df )
 
+def knn_clustering_alignment( P , Q ) :
+    
+    NOTE_ = "This is just a standard kmeans in arbitrary dimensions that start out with centroids that have been shape aligned"
+    ispanda = lambda P: 'pandas' in str(type(P)).lower()
+    BustedPanda = lambda R : R.values if ispanda(R) else R
+    P_ = BustedPanda ( P )
+    Q_ = BustedPanda ( Q )
+
+    centroids = ifit .ShapeAlignment ( P_ , Q_ ,
+                    bReturnTransform = False ,
+                    bShiftModel      = True  ,
+                    bUnrestricted    = True  )
+    
+    if ispanda ( Q ) :
+        #
+        # FOR DIAGNOSTIC PURPOSES
+        centroids_df = pd.DataFrame ( centroids ,
+        index = Q.index ,
+        columns = Q.columns )
+        lookup_ = {i:n for n,i in zip( centroids_df.index,range(len(centroids_df.index)) ) }
+
+    labels , centroids = icluster.seeded_kmeans( P_ , centroids )
+
+    if ispanda ( Q ) and ispanda ( P ) :
+        #
+        # MORE DIAGNOSTICS
+        res_df = pd.DataFrame( [labels] , columns=P.index , index=['cluster index'] )
+        res_df .loc[ 'cluster name' ] = [ lookup_[l] for l in res_df.loc['cluster index'].values ]
+        print ( res_df )
+
+    return ( np.array(labels), np.array(centroids) )
+
+
+
 
 crop = lambda x,W:x[:,:W]
 def run_shape_alignment_regression( analyte_df , journal_df , formula ,
@@ -501,20 +534,28 @@ def glm_test (  formula , df , jdf , distribution='Gaussian' ) :
 
 def t_test ( df , endogen = 'expression' , group = 'disease' ,
              pair_values = ('Sick','Healthy') , test_type = 'independent',
-             equal_var = False , alternative = 'greater' ) :
-    group1 = df[df[group] == pair_values[0]][endogen].astype(float)
-    group2 = df[df[group] == pair_values[1]][endogen].astype(float)
-    if test_type == 'independent':
-        pv = ttest_ind( group1, group2 , equal_var = equal_var )
-    if test_type == 'related':
-        pv = ttest_rel( group1, group2 )
+             equal_var = False , alternative = 'greater' ,
+             bDeprecated = False ) :
+
+    if bDeprecated :
+        print ( 'WILL BE REMOVED IN FUTURE VERSIONS' ) 
+        group1 = df[df[group] == pair_values[0]][endogen].astype(float)
+        group2 = df[df[group] == pair_values[1]][endogen].astype(float)
+    else :
+        group1 = df.iloc[:,[n in pair_values[0] for n in df.loc[group,:].values] ].loc[endogen,:].astype(float)
+        group2 = df.iloc[:,[n in pair_values[1] for n in df.loc[group,:].values] ].loc[endogen,:].astype(float)
+
+    if test_type == 'independent' :
+        pv = ttest_ind ( group1, group2 , equal_var = equal_var )
+    if test_type == 'related' :
+        pv = ttest_rel ( group1, group2 )
     try :
-        p_normality = mannwhitneyu( group1, group2, alternative=alternative )[1]
+        p_mannu = mannwhitneyu( group1, group2, alternative=alternative )[1]
     except ValueError as err:
         print(err.args)
-        p_normality = 1.0
+        p_mannu = 1.0
     pvalue = pv[1] ; statistic=pv[0]
-    return ( pvalue , p_normality, statistic )
+    return ( pvalue , p_mannu, statistic )
 
 def mycov( x , full_matrices=0 ):
     x = x - x.mean( axis=0 )
@@ -689,13 +730,15 @@ def quantify_groups_by_analyte_pvalues( analyte_df, grouping_file, delimiter='\t
             edf.loc[l] = q
     return ( edf.T )
 
+
+
 class APCA ( object ) :
     #
-    # THIS CLASS PERFORMS A SPARSE PCA
-    # IT USES THE SPARSE SVD ALGORITHM
-    # FOUND IN SCIPY
+    # THIS CLASS PERFORMS A SPARSE PCA IF REQUESTED
+    # IT THEN USES THE SPARSE SVD ALGORITHM FOUND IN SCIPY
+    # THE STANDARD IS TO USE THE NUMPY SVD
     #
-    def __init__ ( self , X=None , k=-1 , fillna=None , transcending=True) :
+    def __init__ ( self , X = None , k =-1 , fillna = None , transcending = True , not_sparse = True ) :
         from scipy.sparse import csc_matrix
         from scipy.sparse.linalg import svds
         self.svds_ , self.smatrix_ = svds , csc_matrix
@@ -708,6 +751,7 @@ class APCA ( object ) :
         self.X_   = self.interpret_input(X)
         self.k_   = k
         self.transcending_ = transcending
+        self.not_sparse = not_sparse
 
     def interpret_input ( self,X ) :
         if 'pandas' in str(type(X)) :
@@ -733,7 +777,13 @@ class APCA ( object ) :
             k_ = np.min( np.shape(Xc) ) - 1
         else:
             k_ = self.k_
-        u, s, v = self.svds_ ( self.smatrix_(Xc, dtype=float) , k=k_ )
+
+        if self.not_sparse :
+                u, s, v = np.linalg.svd( Xc , full_matrices = False )
+                self.transcending_ = False
+        else :
+                u, s, v = self.svds_ ( self.smatrix_(Xc, dtype=float) , k=k_ )
+
         if self.transcending_ :
             u, s, v = self.transcending_order(u,s,v)
         S = np.diag( s )
@@ -952,6 +1002,7 @@ def group_counts( analyte_df, grouping_file, delimiter = '\t',
                     eval_df = pd.concat ( [eval_df,ndf] )
     return ( eval_df.sort_values( group_prefix + 'FsigFrom' + p_label ) )
 
+
 def retrieve_genes_of ( group_name, grouping_file, delimiter='\t', identifier='ENSG', skip_line_char='#' ):
     all_analytes = []
     with open ( grouping_file ) as input:
@@ -990,6 +1041,7 @@ def differential_analytes ( analyte_df , cols = [['a'],['b']] ):
     ddf = ddf.sort_values('Dist', ascending = False )
     return ( ddf )
 
+
 def add_kendalltau( analyte_results_df , journal_df , what='M' , sample_names = None, ForceSpearman=False ) :
     # ADD IN CONCORDANCE WITH KENDALL TAU
     if what in set(journal_df.index.values) :
@@ -1014,6 +1066,70 @@ def add_kendalltau( analyte_results_df , journal_df , what='M' , sample_names =
     return ( analyte_results_df )
 
 
+def calculate_rates( journal_df , inferred_df ,
+                     formula , inference_label = 'owner',
+                     bVerbose=False ) :
+
+    check = []
+    for idx in journal_df.index.values:
+        if idx in formula :
+            check.append( idx )
+    check = list( set(check) )
+    if len( check ) == 0 :
+        print( 'Cannot assert quality' )
+        results_lookup = dict( )
+        return ( results_lookup )
+   
+    known_df = journal_df.loc[check,:]
+    
+    known_OH = create_encoding_journal( known_df.index.values, known_df )
+    known_instances = known_OH.index
+    inferred_OH = known_OH*0
+    for o in inferred_df.iterrows():
+        for known_instance in known_instances:
+            inferred_OH.loc[known_instance,o[0]] = int( known_instance in o[1][inference_label] )
+            
+    OH_not = lambda df:df.apply( lambda x:1-x )
+
+    not_known_OH    = OH_not( known_OH )
+    not_inferred_OH = OH_not(inferred_OH)
+
+    if bVerbose:
+        print(known_OH.iloc[:6,:6])
+        print(not_known_OH.iloc[:6,:6])
+        print(inferred_OH.iloc[:6,:6])
+        print(np.sum(np.sum(inferred_OH.iloc[:6,:6] * known_OH.iloc[:6,:6] )) )
+
+    TP = np.sum( np.sum( inferred_OH     * known_OH     ) )
+    FP = np.sum( np.sum( inferred_OH     * not_known_OH ) )
+    TN = np.sum( np.sum( not_inferred_OH * not_known_OH ) )
+    FN = np.sum( np.sum( not_inferred_OH * known_OH     ) )
+
+    results_lookup = {  'TP':TP , 'TN':TN ,
+                'FN':FN , 'FP':FP ,
+                'sensitivity' : TP / ( TP+FN ) ,
+                'specificity' : TN / ( TN+FP ) ,
+                'precision'   : TP / ( TP+FP ) ,
+                'accuracy'    : ( TP+TN ) / ( TP+TN+FP+FN ) ,
+                'negation'    : TN / ( TN+FN ) , # FNR
+                'FPR'         : FP / ( FP+TN ) , # False positive rate
+                'FDR'         : FP / ( FP+TP )   # False discovery rate
+    }
+    return ( results_lookup )
+
+
+def assign_quality_measures( journal_df , result_dfs ,
+                             formula , inference_label='owner' ,
+                             plabel = ',p' , qlabel = ',q' ) :
+
+    for label in [ col for col in result_dfs[0].columns if plabel in col[-2:] ] :
+        result_dfs[0].loc[:, label[:-2]+',q'] = [ qvs[0] for qvs in qvalues( result_dfs[0].loc[:,label].values ) ]
+   
+    results_lookup = calculate_rates ( journal_df , result_dfs[1] ,
+                          formula , inference_label = inference_label )
+    return( results_lookup )
+
+
 if __name__ == '__main__' :
 
     test_type = 'random'
